@@ -68,7 +68,7 @@ enum keyword_numeric_config_field {
     KBF_FORWARDCONTROL,
     KBF_OVERRIDEMTU,
     KBF_STRICTCRLPOLICY,
-    KBF_NOCRSEND,
+    KBF_NOCRSEND,     /* 10 */
     KBF_NATTRAVERSAL,
     KBF_KEEPALIVE,
     KBF_PLUTORESTARTONCRASH,
@@ -78,8 +78,8 @@ enum keyword_numeric_config_field {
     KBF_PLUTODEBUG,
     KBF_NHELPERS,
     KBF_OPPOENCRYPT,
-    KBF_DPDDELAY,
-    KBF_DPDTIMEOUT,
+    KBF_DPDDELAY,     /* 20 */
+    KBF_DPDTIMEOUT,   
     KBF_PHASE2,
     KBF_AUTHBY,
     KBF_KEYEXCHANGE,
@@ -88,14 +88,15 @@ enum keyword_numeric_config_field {
     KBF_SALIFETIME,
     KBF_REKEY,
     KBF_REKEYMARGIN,
-    KBF_REKEYFUZZ,
+    KBF_REKEYFUZZ,   /* 30 */
     KBF_COMPRESS,
-    KBF_KEYINGTRIES,
+    KBF_KEYINGTRIES, 
     KBF_ARRIVALCHECK,
     KBF_IKELIFETIME,
     KBF_AGGRMODE,
     KBF_MODECONFIGPULL,
     KBF_FORCEENCAP,
+    KBF_IKEv2,       /* 38 */
     KBF_MAX         
 };
 
@@ -127,6 +128,10 @@ enum keyword_string_conn_field {
     KSCF_SOURCEIP     = 15,
     KSCF_XAUTHUSERNAME= 16,
     KSCF_SUBNETS      = 17,
+    KSCF_MODECFGDNS1  = 18,
+    KSCF_MODECFGDNS2  = 19,
+    KSCF_MODECFGWINS1 = 20,
+    KSCF_MODECFGWINS2 = 21,
     KSCF_MAX          
 };
 
@@ -236,11 +241,22 @@ struct kw_list {
     unsigned int number;
 };
 
+struct starter_comments {
+    TAILQ_ENTRY(starter_comments) link;
+    char *x_comment;
+    char *commentvalue;
+};
+
+TAILQ_HEAD(starter_comments_list, starter_comments);
+extern void move_comment_list(struct starter_comments_list *to,
+			      struct starter_comments_list *from);
+
 struct section_list {
     TAILQ_ENTRY(section_list) link;
 
     char *name;    
     struct kw_list *kw;
+    struct starter_comments_list comments;
     bool  beenhere;
 };
 
@@ -250,6 +266,8 @@ struct config_parsed {
     TAILQ_HEAD(sectionhead, section_list) sections;
     int ipsec_conf_version;
 
+    struct starter_comments_list comments;
+
     struct section_list conn_default;
     bool                got_default;
 };
