@@ -57,6 +57,20 @@ struct keyword_enum_value kw_keyexchange_values[]={
 struct keyword_enum_values kw_keyexchange_list=
     { kw_keyexchange_values, sizeof(kw_keyexchange_values)/sizeof(struct keyword_enum_value)};
 
+/*
+ * Values for Four-State options, such as ikev2
+ */
+struct keyword_enum_value kw_fourvalued_values[]={
+    { "never",     fo_never  },
+    { "permit",    fo_permit },
+    { "propose",   fo_propose},
+    { "insist",    fo_insist },
+    { "yes",       fo_propose},
+    { "always",    fo_insist },
+    { "no",        fo_never  }
+};
+struct keyword_enum_values kw_fourvalued_list=
+{ kw_fourvalued_values, sizeof(kw_fourvalued_values)/sizeof(struct keyword_enum_value)};
 
 /*
  * Values for authby={rsasig, secret}
@@ -236,7 +250,6 @@ struct keyword_enum_value kw_sendcert_values[]={
 struct keyword_enum_values kw_sendcert_list=
     { kw_sendcert_values, sizeof(kw_sendcert_values)/sizeof(struct keyword_enum_value)};
 
-
 /* MASTER KEYWORD LIST */
 struct keyword_def ipsec_conf_keywords_v2[]={
     {"interfaces",     kv_config, kt_string,    KSF_INTERFACES,NOT_ENUM},
@@ -296,6 +309,7 @@ struct keyword_def ipsec_conf_keywords_v2[]={
     {"type",           kv_conn,         kt_enum,   KBF_TYPE,        &kw_type_list},
     {"authby",         kv_conn|kv_auto, kt_enum,   KBF_AUTHBY,     &kw_authby_list},
     {"keyexchange",    kv_conn|kv_auto, kt_enum,   KBF_KEYEXCHANGE, &kw_keyexchange_list},
+    {"ikev2",          kv_conn|kv_auto|kv_processed,kt_enum,KBF_IKEv2,&kw_fourvalued_list},
     {"pfs",            kv_conn|kv_auto, kt_bool,   KBF_PFS,          NOT_ENUM},
     {"keylife",        kv_conn|kv_auto|kv_alias, kt_time,   KBF_SALIFETIME,NOT_ENUM},
     {"lifetime",       kv_conn|kv_auto|kv_alias, kt_time,   KBF_SALIFETIME,NOT_ENUM},
@@ -336,6 +350,10 @@ struct keyword_def ipsec_conf_keywords_v2[]={
     {"modecfgclient", kv_conn|kv_auto|kv_leftright, kt_bool, KNCF_MODECONFIGCLIENT, NOT_ENUM},
     {"xauthusername", kv_conn|kv_auto|kv_leftright, kt_string, KSCF_XAUTHUSERNAME, NOT_ENUM},
     {"modecfgpull", kv_conn|kv_auto, kt_invertbool, KBF_MODECONFIGPULL , NOT_ENUM},
+    {"modecfgdns1", kv_conn|kv_auto|kv_leftright, kt_ipaddr, KSCF_MODECFGDNS1,NOT_ENUM},
+    {"modecfgdns2", kv_conn|kv_auto|kv_leftright, kt_ipaddr, KSCF_MODECFGDNS2,NOT_ENUM},
+    {"modecfgwins1", kv_conn|kv_auto|kv_leftright, kt_ipaddr, KSCF_MODECFGWINS1,NOT_ENUM},
+    {"modecfgwins2", kv_conn|kv_auto|kv_leftright, kt_ipaddr, KSCF_MODECFGWINS2,NOT_ENUM},
     /* things for manual keying only */
     {"spi",            kv_conn|kv_leftright|kv_manual, kt_number, KNCF_SPI,NOT_ENUM},
     {"espenckey",      kv_conn|kv_leftright|kv_manual, kt_bitstring, KSCF_ESPENCKEY,NOT_ENUM},
@@ -344,9 +362,9 @@ struct keyword_def ipsec_conf_keywords_v2[]={
     {NULL, 0, 0, 0, NOT_ENUM}
 };
 
+/* distinguished keyword */
 struct keyword_def ipsec_conf_keyword_comment=
-{"x-comment",   kv_conn|kv_config, kt_comment, 0, NOT_ENUM};
-
+{"x-comment",      kv_conn,   kt_comment, 0, NOT_ENUM};
 
 
 const int ipsec_conf_keywords_v2_count = sizeof(ipsec_conf_keywords_v2)/sizeof(struct keyword_def);
@@ -367,6 +385,7 @@ int parser_find_keyword(const char *s, YYSTYPE *lval)
 
     keyleft=FALSE;
     k = ipsec_conf_keywords_v2;
+
     while(k->keyname != NULL) {
 	if(strcasecmp(s, k->keyname) == 0)
 	{
@@ -392,10 +411,12 @@ int parser_find_keyword(const char *s, YYSTYPE *lval)
 	k++;
     }
 
+    lval->s = NULL;
     /* if we found nothing */
-    if(k->keyname == NULL && s[0]=='x' && s[1]=='-')
+    if(k->keyname == NULL && (s[0]=='x' || s[0]=='X') && s[1]=='-')
     {
 	k = &ipsec_conf_keyword_comment;
+	lval->k.string = strdup(s);
     }
 
     /* if we still found nothing */
@@ -412,6 +433,9 @@ int parser_find_keyword(const char *s, YYSTYPE *lval)
     case kt_time:
 	keywordtype = TIMEWORD;
 	break;
+    case kt_comment:
+	keywordtype = COMMENT;
+	break;
     case kt_bool:
     case kt_invertbool:
 	keywordtype = BOOLWORD;
